# This file was auto-generated by Fern from our API Definition.

import typing

MemoryGetRequestMemoryType = typing.Union[
    typing.Literal["perpetual", "summary_retriever", "message_window"], typing.Any
]
